#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Print.h"
#include "Chars.h"
#include "Sprite.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus()
{
    PrintS(Vram + 1, "SCORE");
    PrintS(Vram + 7, "HI-SCORE");
    PrintS(Vram + 16, "STAGE");
    PrintByteNumber2(Vram + VramRowSize + 18, CurrentStage + 1);
    PrintS(Vram+ 22, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + 27;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                Put2C(pVram, Char_Remain);
                pVram += 2;
                pVram = Put(pVram, Char_Space);
                pVram = Put(pVram, Char_Space);
                pVram += VramRowSize - 2;
                pVram = PrintC(pVram, i + '0');
                pVram = Put(pVram, Char_Space);
            }
            else {
                do {
                    Put2C(pVram, Char_Remain);
                    pVram += 2;
                    --i;
                } while (i > 0);
            }
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize + 9, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize + 23, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 11, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 12, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	C
        0x1d, 0x2b, 0x22, 0x28, 
        0x29, 0x20, 0x1d, 0x1d, 
        0x21, 0x28, 0x1d, 0x27, 
        0x1d, 0x21, 0x22, 0x1e, 
        //	R
        0x29, 0x24, 0x22, 0x28, 
        0x29, 0x20, 0x25, 0x2c, 
        0x29, 0x24, 0x2c, 0x1f, 
        0x21, 0x1e, 0x21, 0x22, 
        //	A
        0x1d, 0x2b, 0x2a, 0x1f, 
        0x29, 0x20, 0x1d, 0x2c, 
        0x29, 0x24, 0x22, 0x2c, 
        0x21, 0x1e, 0x1d, 0x22, 
        //	C
        0x1d, 0x2b, 0x22, 0x28, 
        0x29, 0x20, 0x1d, 0x1d, 
        0x21, 0x28, 0x1d, 0x27, 
        0x1d, 0x21, 0x22, 0x1e, 
        //	K
        0x29, 0x20, 0x25, 0x24, 
        0x29, 0x28, 0x24, 0x1d, 
        0x29, 0x20, 0x2a, 0x1f, 
        0x21, 0x1e, 0x1d, 0x22, 
        //	Y
        0x29, 0x20, 0x29, 0x20, 
        0x21, 0x28, 0x2b, 0x1e, 
        0x1d, 0x29, 0x20, 0x1d, 
        0x1d, 0x21, 0x1e, 0x1d, 
   };

    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + (32 - 6 * 4) / 2;
        p = TitleBytes;
        repeat (6) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 7, "PUSH TRIGGER BUTTON");
    PrintS(Vram + VramRowSize * 23 + 20, "INUFUTO 2023");
}